import {
	ServiceParams,
	App as BaseApp,
	Endpoint,
	GetEndpointParams,
	Operation,
	Language,
	MediaType,
	Charset,
} from '../common';

export interface ClientParams<App extends BaseApp> {
	app: App;
	// todo only select from app's registered languages/media types/charsets
	language?: Language['name'];
	mediaType?: MediaType['name'];
	charset?: Charset['name'];
	fetch?: typeof fetch;
}

export interface ClientConnection {}

export interface Client<App extends BaseApp = BaseApp, Connection extends ClientConnection = ClientConnection> {
	app: App;
	connect(params: ServiceParams): Promise<Connection>;
	disconnect(connection?: Connection): Promise<void>;
	at<TheEndpoint extends Endpoint = Endpoint>(endpoint: TheEndpoint, params?: Record<GetEndpointParams<TheEndpoint>, unknown>): this;
	makeRequest(operation: Operation): ReturnType<typeof fetch>;
}
