import {isTextMediaType} from './media-type';
import Negotiator from 'negotiator';

export interface Charset<Name extends string = string> {
	name: Name;
	encode: (str: string) => Buffer;
	decode: (buf: Buffer) => string;
}

export const FALLBACK_CHARSET = {
	encode: (str: string) => Buffer.from(str, 'utf-8'),
	decode: (buf: Buffer) => buf.toString('utf-8'),
	name: 'utf-8' as const,
} satisfies Charset;

export const getCharset = (availableCharsets: string[], mediaTypeString: string, charset?: string): Charset['name'] | undefined => {
	if (typeof charset === 'undefined') {
		return isTextMediaType(mediaTypeString) ? FALLBACK_CHARSET['name'] : undefined;
	}

	const negotiator = new Negotiator({
		headers: {
			'accept': `${mediaTypeString};charset=${charset}`,
		},
	});

	return negotiator.charset(availableCharsets);
};
