import {App} from './app';
import {getLanguage} from './language';
import {getMediaType, parseAcceptString} from './media-type';
import {getCharset} from './charset';

interface ContentNegotiationHeaders {
	'accept-language'?: string;
	accept?: string;
}

export const getContentNegotiationParams = (app: App, headers: ContentNegotiationHeaders) => {
	const languageString = getLanguage(Array.from(app.languages.keys()), headers['accept-language']);
	const language = (
		typeof languageString !== 'undefined'
			? app.languages.get(languageString)
			: undefined
	);

	const parsedAccept = parseAcceptString(headers['accept']);

	const mediaTypeString = (
		typeof parsedAccept !== 'undefined'
			? getMediaType(Array.from(app.mediaTypes.keys()), parsedAccept.mediaType)
			: undefined
	);
	const mediaType = (
		typeof mediaTypeString !== 'undefined'
			? app.mediaTypes.get(mediaTypeString)
			: undefined
	);

	const charsetString = (
		typeof parsedAccept !== 'undefined' && typeof mediaType !== 'undefined'
			? getCharset(Array.from(app.charsets.keys()), mediaType.name, parsedAccept.params.charset)
			: undefined
	);
	const charset = (
		typeof charsetString !== 'undefined'
			? app.charsets.get(charsetString)
			: undefined
	);

	return {
		language,
		mediaType,
		charset,
	};
};
