import {ImplementationFunction, DataSource} from '@modal-sh/yasumi/backend';
import {operation as defineOperation, fromUrlSearchParams} from '@modal-sh/yasumi';
import {
	DataSourceMethodNotImplementedResponseError,
	ResourceNotFoundResponseError,
	ResourceCollectionFetchedResponse,
	ResourceItemFetchedResponse,
} from '../response';

export const name = 'fetch' as const;

export const method = 'GET' as const;

export const operation = defineOperation({
	name,
	method,
});

export const implementation: ImplementationFunction = async (ctx) => {
	const {
		params,
		endpoint,
		dataSource,
		language,
		query,
	} = ctx;
	// need to genericise the response here so that we don't depend on the HTTP responses.
	const { resourceId } = params;
	const effectiveDataSource: DataSource = endpoint.dataSource ?? dataSource ?? {} as DataSource;
	const { getById, getMultiple } = effectiveDataSource;

	if (typeof resourceId === 'undefined') {
		if (typeof getMultiple === 'undefined') {
			throw new DataSourceMethodNotImplementedResponseError(language.statusMessages.dataSourceMethodNotImplemented);
		}

		const dataSourceQuery = typeof query !== 'undefined' ? fromUrlSearchParams(query) : undefined;
		const items = await getMultiple(dataSourceQuery);
		return new ResourceCollectionFetchedResponse({
			statusMessage: language.statusMessages.resourceCollectionFetched,
			body: items,
		});
	}

	if (typeof getById === 'undefined') {
		throw new DataSourceMethodNotImplementedResponseError(language.statusMessages.dataSourceMethodNotImplemented);
	}

	const item = await getById(resourceId);
	if (!item) {
		throw new ResourceNotFoundResponseError(language.statusMessages.resourceNotFound);
	}

	return new ResourceItemFetchedResponse({
		statusMessage: language.statusMessages.resourceFetched,
		body: item,
	});
};
